package de.kompf.javaxml;

import static org.junit.Assert.*;

import java.io.*;
import java.lang.reflect.*;

import javax.xml.parsers.*;
import javax.xml.transform.TransformerException;

import org.junit.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

/**
 * Test case for {@link GpxDOMEditor}.
 * It demonstrates the use of invocation proxy.
 *
 */
public class GpxDOMEditorTest {
  
  /**
   * Interface that mimics the class under test.
   * Its purpose is to expose the private methods of the CuT
   * in conjunction with {@link PrivateMethodInvocationHandler}.
   */
  interface GpxDOMEditorProxy {
    Document parseFile(File file) throws ParserConfigurationException, SAXException, IOException;
    void writeFile(Document doc, File file) throws TransformerException;
    void createRouteFromWaypoints(Document gpxDoc) throws SAXException;
    void copyAttributes(Element src, Element dst);
    Element createElementWithText(Document doc, String tagName, String text);
  }
  
  private GpxDOMEditorProxy domEditorProxy;
  
  /**
   * Before each test:
   * Create the CuT and the proxy to expose its private methods to the test.
   */
  @Before
  public void createProxy() {
    InvocationHandler handler = new PrivateMethodInvocationHandler(new GpxDOMEditor());
    domEditorProxy = (GpxDOMEditorProxy) Proxy.newProxyInstance(GpxDOMEditorProxy.class.getClassLoader(),
        new Class[] { GpxDOMEditorProxy.class },
        handler);
  }

  /**
   * Test the method createElementWithText.
   */
  @Test
  public void testCreateElementWithText() throws Exception {
    Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
    final String tagName = "TestElement";
    final String text = "Test <Text> ÄÖÜß";
    Element ele = domEditorProxy.createElementWithText(doc, tagName, text);
    assertNotNull(ele);
    assertEquals(tagName, ele.getNodeName());
    NodeList children = ele.getChildNodes();
    assertEquals(1, children.getLength());
    Node childNode = children.item(0);
    assertEquals(Node.TEXT_NODE, childNode.getNodeType());
    assertEquals(text, ((Text) childNode).getData());
  }
}
