package de.kompf.javaxml;

import java.lang.reflect.*;

/**
 * Invocation handler to expose the private methods of an object.
 */
public class PrivateMethodInvocationHandler implements InvocationHandler {
  
  private Object wrappedObject;
  private Class<?> wrappedClass;

  /**
   * Create a new invocation handler.
   * @param wrappedObject The object which private methods should be exposed.
   */
  public PrivateMethodInvocationHandler(Object wrappedObject) {
    this.wrappedObject = wrappedObject;
    this.wrappedClass = wrappedObject.getClass();
  }

  @Override
  public Object invoke(Object proxy, Method method, Object[] args)
      throws Throwable {
    Method wrappedMethod = wrappedClass.getDeclaredMethod(method.getName(), method.getParameterTypes());
    wrappedMethod.setAccessible(true);
    
    return wrappedMethod.invoke(wrappedObject, args);
  }
}
